@@ -1,3 +1,10 @@
+"""
+$description A state/privately owned Russian live TV channel.
+$url 1tv.ru
+$type live
+$region Russia
+"""
+
 import logging
 import random
 import re
@@ -5,9 +12,7 @@ from urllib.parse import unquote
 
 from streamlink.plugin import Plugin, pluginmatcher
 from streamlink.plugin.api import validate
-from streamlink.plugin.plugin import stream_weight
-from streamlink.stream import HLSStream
-from streamlink.utils import parse_json
+from streamlink.stream.hls import HLSStream
 from streamlink.utils.url import update_qsd
 
 log = logging.getLogger(__name__)
@@ -16,25 +21,31 @@ log = logging.getLogger(__name__)
 @pluginmatcher(re.compile(
     r"https?://(?:www\.)?1tv\.ru/live"
 ))
+@pluginmatcher(re.compile(
+    r"https?://static\.1tv\.ru/eump/(?:embeds|pages)/1tv_live(?:_orbit-plus-4)?\.html"
+))
 class OneTV(Plugin):
-    @classmethod
-    def stream_weight(cls, stream):
-        return dict(ld=(140, "pixels"), sd=(360, "pixels"), hd=(720, "pixels")).get(stream, stream_weight(stream))
-
     def _get_streams(self):
+        if "orbit-plus-4" in self.url:
+            channel = "1tv-orbit-plus-4"
+            self.title = "Первый канал HD (+4)"
+        else:
+            channel = "1tvch"
+            self.title = "Первый канал HD"
+
         url = self.session.http.get(
-            "https://stream.1tv.ru/api/playlist/1tvch_as_array.json",
+            f"https://stream.1tv.ru/api/playlist/{channel}_as_array.json",
             data={"r": random.randint(1, 100000)},
             schema=validate.Schema(
-                validate.transform(parse_json),
+                validate.parse_json(),
                 {"hls": [validate.url()]},
-                validate.get("hls"),
-                validate.get(0),
+                validate.get(("hls", 0)),
             ))
 
         if not url:
             return
 
+        log.debug(f"{url}")
         if "georestrictions" in url:
             log.error("Stream is geo-restricted")
             return
@@ -42,12 +53,12 @@ class OneTV(Plugin):
         hls_session = self.session.http.get(
             "https://stream.1tv.ru/get_hls_session",
             schema=validate.Schema(
-                validate.transform(parse_json),
+                validate.parse_json(),
                 {"s": validate.transform(unquote)},
             ))
         url = update_qsd(url, qsd=hls_session, safe="/:")
 
-        yield from HLSStream.parse_variant_playlist(self.session, url, name_fmt="{pixels}_{bitrate}").items()
+        return HLSStream.parse_variant_playlist(self.session, url, name_fmt="{pixels}_{bitrate}")
 
 
 __plugin__ = OneTV
