@@ -1,5 +1,5 @@
 """
-$description Global live streaming and video on-demand platform owned by IBM.
+$description Global live-streaming and video on-demand platform owned by IBM.
 $url ustream.tv
 $url video.ibm.com
 $type live, vod
@@ -11,13 +11,13 @@ from collections import deque
 from datetime import datetime, timedelta
 from random import randint
 from threading import Event, RLock
-from typing import Any, Callable, Deque, Dict, List, NamedTuple, Union
+from typing import Any, Callable, Deque, Dict, List, NamedTuple, Optional, Union
 from urllib.parse import urljoin, urlunparse
 
 from requests import Response
 
 from streamlink.exceptions import PluginError, StreamError
-from streamlink.plugin import Plugin, PluginArgument, PluginArguments, pluginmatcher
+from streamlink.plugin import Plugin, pluginargument, pluginmatcher
 from streamlink.plugin.api import useragents, validate
 from streamlink.plugin.api.websocket import WebsocketClient
 from streamlink.stream.ffmpegmux import MuxedStream
@@ -56,9 +56,9 @@ class Segment(NamedTuple):
     path: str
 
     # the segment URLs depend on the CDN and the chosen stream format and its segment template string
-    def url(self, base: str, template: str) -> str:
+    def url(self, base: Optional[str], template: str) -> str:
         return urljoin(
-            base,
+            base or "",
             f"{self.path}/{template.replace('%', str(self.num), 1).replace('%', self.hash, 1)}"
         )
 
@@ -117,10 +117,10 @@ class UStreamTVWsClient(WebsocketClient):
         "hashes": {validate.transform(int): str}
     })
 
-    stream_cdn: str = None
-    stream_formats_video: List[StreamFormatVideo] = None
-    stream_formats_audio: List[StreamFormatAudio] = None
-    stream_initial_id: int = None
+    stream_cdn: Optional[str] = None
+    stream_formats_video: Optional[List[StreamFormatVideo]] = None
+    stream_formats_audio: Optional[List[StreamFormatAudio]] = None
+    stream_initial_id: Optional[int] = None
 
     def __init__(
         self,
@@ -213,8 +213,8 @@ class UStreamTVWsClient(WebsocketClient):
 
         cmd: str = parsed["cmd"]
         args: List[Dict] = parsed["args"]
-        log.trace(f"Received '{cmd}' command")
-        log.trace(f"{args!r}")
+        log.trace(f"Received '{cmd}' command")  # type: ignore[attr-defined]
+        log.trace(f"{args!r}")  # type: ignore[attr-defined]
 
         handlers = self._MESSAGE_HANDLERS.get(cmd)
         if handlers is not None:
@@ -338,8 +338,8 @@ class UStreamTVWsClient(WebsocketClient):
 
 
 class UStreamTVStreamWriter(SegmentedStreamWriter):
-    stream: "UStreamTVStream"
     reader: "UStreamTVStreamReader"
+    stream: "UStreamTVStream"
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -358,7 +358,7 @@ class UStreamTVStreamWriter(SegmentedStreamWriter):
             self.queue(segment, self.executor.submit(self.fetch, segment, False))
 
     # noinspection PyMethodOverriding
-    def fetch(self, segment: Segment, is_init: bool):
+    def fetch(self, segment: Segment, is_init: bool):  # type: ignore[override]
         if self.closed:  # pragma: no cover
             return
 
@@ -394,6 +394,8 @@ class UStreamTVStreamWriter(SegmentedStreamWriter):
 
 
 class UStreamTVStreamWorker(SegmentedStreamWorker):
+    reader: "UStreamTVStreamReader"
+    writer: "UStreamTVStreamWriter"
     stream: "UStreamTVStream"
 
     def __init__(self, *args, **kwargs):
@@ -428,7 +430,10 @@ class UStreamTVStreamWorker(SegmentedStreamWorker):
 class UStreamTVStreamReader(SegmentedStreamReader):
     __worker__ = UStreamTVStreamWorker
     __writer__ = UStreamTVStreamWriter
+
     stream: "UStreamTVStream"
+    worker: "UStreamTVStreamWorker"
+    writer: "UStreamTVStreamWriter"
 
     def open(self):
         self.stream.wsclient.opened.set()
@@ -462,33 +467,32 @@ class UStreamTVStream(Stream):
 
 
 @pluginmatcher(re.compile(r"""
-    https?://(?:(www\.)?ustream\.tv|video\.ibm\.com)
-        (?:
-            (/embed/|/channel/id/)(?P<channel_id>\d+)
-        )?
-        (?:
-            (/embed)?/recorded/(?P<video_id>\d+)
-        )?
-""", re.VERBOSE))
-class UStreamTV(Plugin):
-    arguments = PluginArguments(
-        PluginArgument(
-            "password",
-            argument_name="ustream-password",
-            sensitive=True,
-            metavar="PASSWORD",
-            help="A password to access password protected UStream.tv channels."
-        )
+    https?://(?:(?:www\.)?ustream\.tv|video\.ibm\.com)
+    (?:
+        /combined-embed
+        /(?P<combined_channel_id>\d+)
+        (?:/video/(?P<combined_video_id>\d+))?
+        |
+        (?:(?:/embed/|/channel/(?:id/)?)(?P<channel_id>\d+))?
+        (?:(?:/embed)?/recorded/(?P<video_id>\d+))?
     )
-
+""", re.VERBOSE))
+@pluginargument(
+    "password",
+    sensitive=True,
+    argument_name="ustream-password",
+    metavar="PASSWORD",
+    help="A password to access password protected UStream.tv channels.",
+)
+class UStreamTV(Plugin):
     STREAM_READY_TIMEOUT = 15
 
     def _get_media_app(self):
-        video_id = self.match.group("video_id")
+        video_id = self.match.group("video_id") or self.match.group("combined_video_id")
         if video_id:
             return video_id, "recorded"
 
-        channel_id = self.match.group("channel_id")
+        channel_id = self.match.group("channel_id") or self.match.group("combined_channel_id")
         if not channel_id:
             channel_id = self.session.http.get(
                 self.url,
