@@ -2,11 +2,12 @@ import logging
 import pkgutil
 from functools import lru_cache
 from socket import AF_INET, AF_INET6
-from typing import Any, Dict, Optional, Tuple, Type
+from typing import Any, Dict, Iterator, Optional, Tuple, Type
 
-import requests
-import requests.packages.urllib3.util.connection as urllib3_connection
-from requests.packages.urllib3.util.connection import allowed_gai_family
+# noinspection PyPackageRequirements
+import urllib3.util.connection as urllib3_util_connection
+# noinspection PyPackageRequirements
+import urllib3.util.ssl_ as urllib3_util_ssl
 
 from streamlink import __version__, plugins
 from streamlink.exceptions import NoPluginError, PluginError
@@ -23,6 +24,22 @@ logging.setLoggerClass(StreamlinkLogger)
 log = logging.getLogger(__name__)
 
 
+# noinspection PyUnresolvedReferences
+_original_allowed_gai_family = urllib3_util_connection.allowed_gai_family  # type: ignore[attr-defined]
+
+# options which support `key1=value1;key2=value2;...` strings as value
+_OPTIONS_HTTP_KEYEQUALSVALUE = {"http-cookies": "cookies", "http-headers": "headers", "http-query-params": "params"}
+
+
+def _parse_keyvalue_string(value: str) -> Iterator[Tuple[str, str]]:
+    for keyval in value.split(";"):
+        try:
+            key, val = keyval.split("=", 1)
+            yield key.strip(), val.strip()
+        except ValueError:
+            continue
+
+
 class PythonDeprecatedWarning(UserWarning):
     pass
 
@@ -71,7 +88,7 @@ class Streamlink:
             "ffmpeg-start-at-zero": False,
             "mux-subtitles": False,
             "locale": None,
-            "user-input-requester": None
+            "user-input-requester": None,
         })
         if options:
             self.options.update(options)
@@ -133,6 +150,8 @@ class Streamlink:
         http-ssl-verify          (bool) Verify SSL certificates,
                                  default: ``True``
 
+        http-disable-dh          (bool) Disable SSL Diffie-Hellman key exchange
+
         http-ssl-cert            (str or tuple) SSL certificate to use,
                                  can be either a .pem file (str) or a
                                  .crt/.key pair (tuple)
@@ -193,9 +212,8 @@ class Streamlink:
                                  default: ``system locale``.
 
         user-input-requester     (UserInputRequester) instance of UserInputRequester
-                                 to collect input from the user at runtime. Must be
-                                 set before the plugins are loaded.
-                                 default: ``UserInputRequester``.
+                                 to collect input from the user at runtime.
+                                 default: ``None``.
         ======================== =========================================
         """
 
@@ -214,49 +232,46 @@ class Streamlink:
 
         elif key == "ipv4" or key == "ipv6":
             self.options.set(key, value)
-            if value:
-                self.options.set("ipv6" if key == "ipv4" else "ipv4", False)
-                urllib3_connection.allowed_gai_family = \
-                    (lambda: AF_INET) if key == "ipv4" else (lambda: AF_INET6)
+            if not value:
+                urllib3_util_connection.allowed_gai_family = _original_allowed_gai_family  # type: ignore[attr-defined]
+            elif key == "ipv4":
+                self.options.set("ipv6", False)
+                urllib3_util_connection.allowed_gai_family = (lambda: AF_INET)  # type: ignore[attr-defined]
             else:
-                urllib3_connection.allowed_gai_family = allowed_gai_family
+                self.options.set("ipv4", False)
+                urllib3_util_connection.allowed_gai_family = (lambda: AF_INET6)  # type: ignore[attr-defined]
 
         elif key in ("http-proxy", "https-proxy"):
             self.http.proxies["http"] = update_scheme("https://", value, force=False)
             self.http.proxies["https"] = self.http.proxies["http"]
             if key == "https-proxy":
-                log.info("The https-proxy option has been deprecated in favour of a single http-proxy option")
+                log.warning("The https-proxy option has been deprecated in favor of a single http-proxy option")
+
+        elif key in _OPTIONS_HTTP_KEYEQUALSVALUE:
+            getattr(self.http, _OPTIONS_HTTP_KEYEQUALSVALUE[key]).update(
+                value if isinstance(value, dict) else dict(_parse_keyvalue_string(value))
+            )
 
-        elif key == "http-cookies":
-            if isinstance(value, dict):
-                self.http.cookies.update(value)
-            else:
-                self.http.parse_cookies(value)
-        elif key == "http-headers":
-            if isinstance(value, dict):
-                self.http.headers.update(value)
-            else:
-                self.http.parse_headers(value)
-        elif key == "http-query-params":
-            if isinstance(value, dict):
-                self.http.params.update(value)
-            else:
-                self.http.parse_query_params(value)
         elif key == "http-trust-env":
             self.http.trust_env = value
+
         elif key == "http-ssl-verify":
             self.http.verify = value
+
         elif key == "http-disable-dh":
+            default_ciphers = [
+                item
+                for item in urllib3_util_ssl.DEFAULT_CIPHERS.split(":")  # type: ignore[attr-defined]
+                if item != "!DH"
+            ]
+
             if value:
-                requests.packages.urllib3.util.ssl_.DEFAULT_CIPHERS += ':!DH'
-                try:
-                    requests.packages.urllib3.contrib.pyopenssl.DEFAULT_SSL_CIPHER_LIST = \
-                        requests.packages.urllib3.util.ssl_.DEFAULT_CIPHERS.encode("ascii")
-                except AttributeError:
-                    # no ssl to disable the cipher on
-                    pass
+                default_ciphers.append("!DH")
+            urllib3_util_ssl.DEFAULT_CIPHERS = ":".join(default_ciphers)  # type: ignore[attr-defined]
+
         elif key == "http-ssl-cert":
             self.http.cert = value
+
         elif key == "http-timeout":
             self.http.timeout = value
 
@@ -305,7 +320,7 @@ class Streamlink:
         else:
             return self.options.get(key)
 
-    def set_plugin_option(self, plugin: str, key: str, value: Any):
+    def set_plugin_option(self, plugin: str, key: str, value: Any) -> None:
         """
         Sets plugin specific options used by plugins originating from this session object.
 
@@ -315,10 +330,10 @@ class Streamlink:
         """
 
         if plugin in self.plugins:
-            plugin = self.plugins[plugin]
-            plugin.set_option(key, value)
+            plugincls = self.plugins[plugin]
+            plugincls.set_option(key, value)
 
-    def get_plugin_option(self, plugin: str, key: str):
+    def get_plugin_option(self, plugin: str, key: str) -> Optional[Any]:
         """
         Returns the current value of the plugin specific option.
 
@@ -327,15 +342,15 @@ class Streamlink:
         """
 
         if plugin in self.plugins:
-            plugin = self.plugins[plugin]
-            return plugin.get_option(key)
+            plugincls = self.plugins[plugin]
+            return plugincls.get_option(key)
 
     @lru_cache(maxsize=128)
     def resolve_url(
         self,
         url: str,
-        follow_redirect: bool = True
-    ) -> Tuple[Type[Plugin], str]:
+        follow_redirect: bool = True,
+    ) -> Tuple[str, Type[Plugin], str]:
         """
         Attempts to find a plugin that can use this URL.
 
@@ -351,30 +366,29 @@ class Streamlink:
         url = update_scheme("https://", url, force=False)
 
         matcher: Matcher
-        candidate: Optional[Type[Plugin]] = None
+        candidate: Optional[Tuple[str, Type[Plugin]]] = None
         priority = NO_PRIORITY
         for name, plugin in self.plugins.items():
             if plugin.matchers:
                 for matcher in plugin.matchers:
                     if matcher.priority > priority and matcher.pattern.match(url) is not None:
-                        candidate = plugin
+                        candidate = name, plugin
                         priority = matcher.priority
             # TODO: remove deprecated plugin resolver
             elif hasattr(plugin, "can_handle_url") and callable(plugin.can_handle_url) and plugin.can_handle_url(url):
                 prio = plugin.priority(url) if hasattr(plugin, "priority") and callable(plugin.priority) else NORMAL_PRIORITY
                 if prio > priority:
-                    log.info(f"Resolved plugin {name} with deprecated can_handle_url API")
-                    candidate = plugin
+                    log.warning(f"Resolved plugin {name} with deprecated can_handle_url API")
+                    candidate = name, plugin
                     priority = prio
 
         if candidate:
-            return candidate, url
+            return candidate[0], candidate[1], url
 
         if follow_redirect:
             # Attempt to handle a redirect URL
             try:
-                # noinspection PyArgumentList
-                res = self.http.head(url, allow_redirects=True, acceptable_status=[501])
+                res = self.http.head(url, allow_redirects=True, acceptable_status=[501])  # type: ignore[call-arg]
 
                 # Fall back to GET request if server doesn't handle HEAD.
                 if res.status_code == 501:
@@ -387,7 +401,7 @@ class Streamlink:
 
         raise NoPluginError
 
-    def resolve_url_no_redirect(self, url: str) -> Tuple[Type[Plugin], str]:
+    def resolve_url_no_redirect(self, url: str) -> Tuple[str, Type[Plugin], str]:
         """
         Attempts to find a plugin that can use this URL.
 
@@ -409,8 +423,8 @@ class Streamlink:
         :return: A :class:`dict` of stream names and :class:`streamlink.stream.Stream` instances
         """
 
-        pluginclass, resolved_url = self.resolve_url(url)
-        plugin = pluginclass(resolved_url)
+        pluginname, pluginclass, resolved_url = self.resolve_url(url)
+        plugin = pluginclass(self, resolved_url)
 
         return plugin.streams(**params)
 
@@ -431,9 +445,9 @@ class Streamlink:
         """
 
         success = False
-        user_input_requester = self.get_option("user-input-requester")
         for loader, name, ispkg in pkgutil.iter_modules([path]):
             # set the full plugin module name
+            # use the "streamlink.plugins." prefix even for sideloaded plugins
             module_name = f"streamlink.plugins.{name}"
             try:
                 mod = load_module(module_name, path)
@@ -445,10 +459,9 @@ class Streamlink:
                 continue
             success = True
             plugin = mod.__plugin__
-            plugin.bind(self, name, user_input_requester)
-            if plugin.module in self.plugins:
-                log.debug(f"Plugin {plugin.module} is being overridden by {mod.__file__}")
-            self.plugins[plugin.module] = plugin
+            if name in self.plugins:
+                log.debug(f"Plugin {name} is being overridden by {mod.__file__}")
+            self.plugins[name] = plugin
 
         return success
 
