@@ -3,17 +3,21 @@ import numbers
 import re
 from string import printable
 from textwrap import dedent
+from typing import Any, Callable, Dict, List, Optional, Tuple
 
 from streamlink import __version__ as streamlink_version, logger
-from streamlink.utils.args import (
-    boolean, comma_list, comma_list_filter, filesize, keyvalue, num
-)
+from streamlink.session import Streamlink
+from streamlink.utils.args import boolean, comma_list, comma_list_filter, filesize, keyvalue, num
 from streamlink.utils.times import hours_minutes_seconds
 from streamlink_cli.constants import (
-    DEFAULT_STREAM_METADATA, PLAYER_ARGS_INPUT_DEFAULT, PLAYER_ARGS_INPUT_FALLBACK, STREAM_PASSTHROUGH, SUPPORTED_PLAYERS
+    PLAYER_ARGS_INPUT_DEFAULT,
+    PLAYER_ARGS_INPUT_FALLBACK,
+    STREAM_PASSTHROUGH,
+    SUPPORTED_PLAYERS,
 )
 from streamlink_cli.utils import find_default_player
 
+
 _printable_re = re.compile(r"[{0}]".format(printable))
 _option_re = re.compile(r"""
     (?P<name>[A-z-]+) # A option name, valid characters are A to z and dash.
@@ -26,6 +30,27 @@ _option_re = re.compile(r"""
 
 
 class ArgumentParser(argparse.ArgumentParser):
+    # noinspection PyUnresolvedReferences,PyProtectedMember
+    NESTED_ARGUMENT_GROUPS: Dict[Optional[argparse._ArgumentGroup], List[argparse._ArgumentGroup]]
+
+    def __init__(self, *args, **kwargs):
+        self.NESTED_ARGUMENT_GROUPS = {}
+        super().__init__(*args, **kwargs)
+
+    # noinspection PyUnresolvedReferences,PyProtectedMember
+    def add_argument_group(
+        self,
+        *args,
+        parent: Optional[argparse._ArgumentGroup] = None,
+        **kwargs
+    ) -> argparse._ArgumentGroup:
+        group = super().add_argument_group(*args, **kwargs)
+        if parent not in self.NESTED_ARGUMENT_GROUPS:
+            self.NESTED_ARGUMENT_GROUPS[parent] = [group]
+        else:
+            self.NESTED_ARGUMENT_GROUPS[parent].append(group)
+        return group
+
     def convert_arg_line_to_args(self, line):
         # Strip any non-printable characters that might be in the
         # beginning of the line (e.g. Unicode BOM marker).
@@ -87,16 +112,18 @@ class ArgumentParser(argparse.ArgumentParser):
         # description
         formatter.add_text(self.description)
 
-        def format_group(group):
+        def format_group(parent):
+            if parent not in self.NESTED_ARGUMENT_GROUPS:
+                return
             # positionals, optionals and user-defined groups
-            for action_group in group._action_groups:
+            for action_group in self.NESTED_ARGUMENT_GROUPS[parent]:
                 formatter.start_section(action_group.title)
                 formatter.add_text(action_group.description)
                 formatter.add_arguments(action_group._group_actions)
                 format_group(action_group)
                 formatter.end_section()
 
-        format_group(self)
+        format_group(None)
 
         # epilog
         formatter.add_text(self.epilog)
@@ -118,11 +145,10 @@ class HelpFormatter(argparse.RawDescriptionHelpFormatter):
     def __init__(self, max_help_position=4, *args, **kwargs):
         # A smaller indent for args help.
         kwargs["max_help_position"] = max_help_position
-        argparse.RawDescriptionHelpFormatter.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def _split_lines(self, text, width):
-        text = dedent(text).strip() + "\n\n"
-        return text.splitlines()
+        return f"{dedent(text).strip()}\n\n".splitlines()
 
 
 def build_parser():
@@ -153,7 +179,7 @@ def build_parser():
         help="""
         A URL to attempt to extract streams from.
 
-        Usually, the protocol of http(s) URLs can be omitted ("https://"),
+        Usually, the protocol of http(s) URLs can be omitted (`https://`),
         depending on the implementation of the plugin being used.
 
         Alternatively, the URL can also be specified by using the --url option.
@@ -167,7 +193,7 @@ def build_parser():
         help="""
         Stream to play.
 
-        Use ``best`` or ``worst`` for selecting the highest or lowest available
+        Use `best` or `worst` for selecting the highest or lowest available
         quality.
 
         Fallback streams can be specified by using a comma-separated list:
@@ -218,7 +244,7 @@ def build_parser():
         help="""
         Check if Streamlink has a plugin that can handle the specified URL.
 
-        Returns status code 1 for false and 0 for true.
+        Returns status code `1` for false and `0` for true.
 
         Useful for external scripting.
         """
@@ -250,18 +276,18 @@ def build_parser():
         help="""
         Set the log message threshold.
 
-        Valid levels are: none, error, warning, info, debug, trace
+        Valid levels are: `none`, `error`, `warning`, `info`, `debug`, `trace`
         """
     )
     general.add_argument(
         "--logfile",
         metavar="FILE",
         help="""
-        Append log output to FILE instead of writing to stdout/stderr.
+        Append log output to `FILE` instead of writing to stdout/stderr.
 
-        User prompts and download progress won't be written to FILE.
+        User prompts and download progress won't be written to `FILE`.
 
-        A value of ``-`` will set the file name to an ISO8601-like string
+        A value of `-` (dash) will set the file name to an ISO8601-like string
         and will choose the following default log directories.
 
         Windows:
@@ -283,7 +309,7 @@ def build_parser():
         help="""
         Hide all log output.
 
-        Alias for "--loglevel none".
+        Alias for `--loglevel none`.
         """
     )
     general.add_argument(
@@ -321,8 +347,8 @@ def build_parser():
         The preferred locale setting, for selecting the preferred subtitle and
         audio language.
 
-        The locale is formatted as [language_code]_[country_code], eg. en_US or
-        es_ES.
+        The locale is formatted as `[language_code]_[country_code]`, eg. `en_US` or
+        `es_ES`.
 
         Default is system locale.
         """
@@ -339,14 +365,14 @@ def build_parser():
         "-4", "--ipv4",
         action="store_true",
         help="""
-        Resolve address names to IPv4 only. This option overrides :option:`-6`.
+        Resolve address names to IPv4 only. This option overrides --ipv6.
         """
     )
     general.add_argument(
         "-6", "--ipv6",
         action="store_true",
         help="""
-        Resolve address names to IPv6 only. This option overrides :option:`-4`.
+        Resolve address names to IPv6 only. This option overrides --ipv4.
         """
     )
 
@@ -396,14 +422,14 @@ def build_parser():
         any of the player arguments to be logged.
 
         The value can contain formatting variables surrounded by curly braces,
-        {{ and }}. If you need to include a brace character, it can be escaped
-        by doubling, e.g. {{{{ and }}}}.
+        `{{` and `}}`. If you need to include a brace character, it can be escaped
+        by doubling, e.g. `{{{{` and `}}}}`.
 
         Formatting variables available:
 
         {{{PLAYER_ARGS_INPUT_DEFAULT}}}
             This is the input that the player will use. For standard input (stdin),
-            it is ``-``, but it can also be a URL, depending on the options used.
+            it is `-` (dash), but it can also be a URL, depending on the options used.
 
         {{{PLAYER_ARGS_INPUT_FALLBACK}}}
             The old fallback variable name with the same functionality.
@@ -412,7 +438,7 @@ def build_parser():
 
           %(prog)s -p vlc -a "--play-and-exit {{{PLAYER_ARGS_INPUT_DEFAULT}}}" <url> [stream]
 
-        Note: When neither of the variables are found, ``{{{PLAYER_ARGS_INPUT_DEFAULT}}}``
+        Note: When neither of the variables are found, `{{{PLAYER_ARGS_INPUT_DEFAULT}}}`
         will be appended to the whole parameter value, to ensure that the player
         always receives an input argument.
         """
@@ -459,14 +485,33 @@ def build_parser():
         useful to allow external devices like smartphones or streaming boxes to
         watch streams they wouldn't be able to otherwise.
 
-        Behavior will be similar to the continuous HTTP option, but no player
-        program will be started, and the server will listen on all available
+        The default behavior is similar to the --player-continuous-http option,
+        but no player program will be started, and the server will listen on all available
         connections instead of just in the local (loopback) interface.
 
+        Optionally, the --player-external-http-continuous option allows for disabling
+        the continuous run-mode, so that Streamlink will stop when the stream ends.
+
         The URLs that can be used to access the stream will be printed to the
         console, and the server can be interrupted using CTRL-C.
         """
     )
+    player.add_argument(
+        "--player-external-http-continuous",
+        type=boolean,
+        metavar="{yes,true,1,on,no,false,0,off}",
+        default=True,
+        help="""
+        Set the run-mode of --player-external-http to continuous or non-continuous.
+
+        In the continuous run-mode, Streamlink will keep running after the stream has ended
+        and will wait for the next HTTP request being made unless it gets shut down via CTRL-C.
+
+        If set to non-continuous, Streamlink will stop once the stream has ended.
+
+        Default is true.
+        """
+    )
     player.add_argument(
         "--player-external-http-port",
         metavar="PORT",
@@ -474,7 +519,7 @@ def build_parser():
         default=0,
         help="""
         A fixed port to use for the external HTTP server if that mode is
-        enabled. Omit or set to 0 to use a random high ( >1024) port.
+        enabled. Omit or set to `0` to use a random high ( >1024) port.
         """
     )
     player.add_argument(
@@ -510,79 +555,27 @@ def build_parser():
     player.add_argument(
         "-t", "--title",
         metavar="TITLE",
-        help="""
-        This option allows you to supply a title to be displayed in the
-        title bar of the window that the video player is launched in.
+        help=f"""
+        Change the title of the video player's window.
 
-        This value can contain formatting variables surrounded by curly braces,
-        {{ and }}. If you need to include a brace character, it can be escaped
-        by doubling, e.g. {{{{ and }}}}.
+        Please see the "Metadata variables" section of Streamlink's CLI documentation for all available metadata variables.
 
-        This option is only supported for the following players: {0}.
+        This option is only supported for the following players: {', '.join(sorted(SUPPORTED_PLAYERS.keys()))}
 
         VLC specific information:
-            VLC has certain codes you can use inside your title.
-            These are accessible inside --title by using a backslash
-            before the dollar sign VLC uses to denote a format character.
-
-            e.g. to put the current date in your VLC window title,
-            the string "\\$A" could be inserted inside your --title string.
-
-            A full list of the format codes VLC uses is available here:
+            VLC does support special formatting variables on its own:
             https://wiki.videolan.org/Documentation:Format_String/
 
-        mpv specific information:
-            mpv has certain codes you can use inside your title.
-            These are accessible inside --title by using a backslash
-            before the dollar sign mpv uses to denote a format character.
+            These variables are accessible in the --title option by adding a backslash
+            in front of the dollar sign which VLC uses as its formatting character.
 
-            e.g. to put the current version of mpv running inside your
-            mpv window title, the string "\\${{{{mpv-version}}}}" could be
-            inserted inside your --title string.
+            For example, to put the current date in your VLC window title,
+            the string `\\$A` could be inserted inside the --title string.
 
-            A full list of the format codes mpv uses is available here:
-            https://mpv.io/manual/stable/#property-list
+        Example:
 
-        Formatting variables available to use in --title:
-
-        {{title}}
-            If available, this is the title of the stream.
-            Otherwise, it is the string "{1}"
-
-        {{author}}
-            If available, this is the author of the stream.
-            Otherwise, it is the string "{2}"
-
-        {{category}}
-            If available, this is the category the stream has been placed into.
-
-            - For Twitch, this is the game being played
-            - For YouTube, it's the category e.g. Gaming, Sports, Music...
-
-            Otherwise, it is the string "{3}"
-
-        {{game}}
-            This is just a synonym for {{category}} which may make more sense for
-            gaming oriented platforms. "Game being played" is a way to categorize
-            the stream, so it doesn't need its own separate handling.
-
-        {{url}}
-            URL of the stream.
-
-        {{time}}
-            The current timestamp, which can optionally be formatted via {{time:format}}.
-            This format parameter string is passed to Python's datetime.strftime() method,
-            so all usual time directives are available. The default format is "%%Y-%%m-%%d_%%H-%%M-%%S".
-
-        Examples:
-
-            %(prog)s -p vlc --title "{{title}} -!- {{author}} -!- {{category}} \\$A" <url> [stream]
-            %(prog)s -p mpv --title "{{title}} -- {{author}} -- {{category}} -- (\\${{{{mpv-version}}}})" <url> [stream]
-
-        """.format(', '.join(sorted(SUPPORTED_PLAYERS.keys())),
-                   DEFAULT_STREAM_METADATA['title'],
-                   DEFAULT_STREAM_METADATA['author'],
-                   DEFAULT_STREAM_METADATA['category'])
+            %(prog)s -p mpv --title "{{author}} - {{category}} - {{title}}" <URL> [STREAM]
+        """
     )
 
     output = parser.add_argument_group("File output options")
@@ -590,26 +583,34 @@ def build_parser():
         "-o", "--output",
         metavar="FILENAME",
         help="""
-        Write stream data to FILENAME instead of playing it.
+        Write stream data to `FILENAME` instead of playing it. If `FILENAME` is set to `-` (dash), then the stream data will be
+        written to stdout, similar to the --stdout argument.
+
+        Non-existent directories and subdirectories will be created if they do not exist, if filesystem permissions allow.
 
         You will be prompted if the file already exists.
 
-        The formatting variables available for the --title option may be used.
+        Please see the "Metadata variables" section of Streamlink's CLI documentation for all available metadata variables.
+
         Unsupported characters in substituted variables will be replaced with an underscore.
+
+        Example:
+
+            %(prog)s --output "~/recordings/{author}/{category}/{id}-{time:%%Y%%m%%d%%H%%M%%S}.ts" <URL> [STREAM]
         """
     )
     output.add_argument(
         "-f", "--force",
         action="store_true",
         help="""
-        When using -o or -r, always write to file even if it already exists.
+        When using --output or --record, always write to file even if it already exists (overwrite).
         """
     )
     output.add_argument(
         "--force-progress",
         action="store_true",
         help="""
-        When using -o or -r,
+        When using --output or --record,
         show the download progress bar even if there is no terminal.
         """
     )
@@ -624,24 +625,39 @@ def build_parser():
         "-r", "--record",
         metavar="FILENAME",
         help="""
-        Open the stream in the player, while at the same time writing it to FILENAME.
+        Open the stream in the player, while at the same time writing it to `FILENAME`. If `FILENAME` is set to `-` (dash),
+        then the stream data will be written to stdout, similar to the --stdout argument, while still opening the player.
+
+        Non-existent directories and subdirectories will be created if they do not exist, if filesystem permissions allow.
 
         You will be prompted if the file already exists.
 
-        The formatting variables available for the --title option may be used.
+        Please see the "Metadata variables" section of Streamlink's CLI documentation for all available metadata variables.
+
         Unsupported characters in substituted variables will be replaced with an underscore.
+
+        Example:
+
+            %(prog)s --record "~/recordings/{author}/{category}/{id}-{time:%%Y%%m%%d%%H%%M%%S}.ts" <URL> [STREAM]
         """
     )
     output.add_argument(
         "-R", "--record-and-pipe",
         metavar="FILENAME",
         help="""
-        Write stream data to stdout, while at the same time writing it to FILENAME.
+        Write stream data to stdout, while at the same time writing it to `FILENAME`.
+
+        Non-existent directories and subdirectories will be created if they do not exist, if filesystem permissions allow.
 
         You will be prompted if the file already exists.
 
-        The formatting variables available for the --title option may be used.
+        Please see the "Metadata variables" section of Streamlink's CLI documentation for all available metadata variables.
+
         Unsupported characters in substituted variables will be replaced with an underscore.
+
+        Example:
+
+            %(prog)s --record-and-pipe "~/recordings/{author}/{category}/{id}-{time:%%Y%%m%%d%%H%%M%%S}.ts" <URL> [STREAM]
         """
     )
     output.add_argument(
@@ -659,8 +675,8 @@ def build_parser():
 
         These characters are replaced with an underscore for the rules in use:
 
-          POSIX  : \\x00-\\x1F /
-          Windows: \\x00-\\x1F \\x7F " * / : < > ? \\ |
+        - POSIX: `\\x00-\\x1F /`
+        - Windows: `\\x00-\\x1F \\x7F " * / : < > ? \\ |`
         """
     )
 
@@ -672,7 +688,7 @@ def build_parser():
         help="""
         A URL to attempt to extract streams from.
 
-        Usually, the protocol of http(s) URLs can be omitted (https://),
+        Usually, the protocol of http(s) URLs can be omitted (`https://`),
         depending on the implementation of the plugin being used.
 
         This is an alternative to setting the URL using a positional argument
@@ -686,7 +702,7 @@ def build_parser():
         help="""
         Stream to play.
 
-        Use ``best`` or ``worst`` for selecting the highest or lowest available
+        Use `best` or `worst` for selecting the highest or lowest available
         quality.
 
         Fallback streams can be specified by using a comma-separated list:
@@ -710,7 +726,7 @@ def build_parser():
         type=num(float, min=0),
         help="""
         Retry fetching the list of available streams until streams are found
-        while waiting DELAY second(s) between each attempt. If unset, only one
+        while waiting `DELAY` second(s) between each attempt. If unset, only one
         attempt will be made to fetch the list of streams available.
 
         The number of fetch retry attempts can be capped with --retry-max.
@@ -721,8 +737,8 @@ def build_parser():
         metavar="COUNT",
         type=num(int, min=-1),
         help="""
-        When using --retry-streams, stop retrying the fetch after COUNT retry
-        attempt(s). Fetch will retry infinitely if COUNT is zero or unset.
+        When using --retry-streams, stop retrying the fetch after `COUNT` retry
+        attempt(s). Fetch will retry infinitely if `COUNT` is zero or unset.
 
         If --retry-max is set without setting --retry-streams, the delay between
         retries will default to 1 second.
@@ -734,7 +750,7 @@ def build_parser():
         type=num(int, min=0),
         default=1,
         help="""
-        After a successful fetch, try ATTEMPTS time(s) to open the stream until
+        After a successful fetch, try `ATTEMPTS` time(s) to open the stream until
         giving up.
 
         Default is 1.
@@ -749,10 +765,10 @@ def build_parser():
 
         The order will be used to separate streams when there are multiple
         streams with the same name but different stream types. Any stream type
-        not listed will be omitted from the available streams list.  A ``*`` can
+        not listed will be omitted from the available streams list.  An `*` (asterisk) can
         be used as a wildcard to match any other type of stream, eg. muxed-stream.
 
-        Default is "rtmp,hls,hds,http,akamaihd,*".
+        Default is "hls,http,*".
         """
     )
     stream.add_argument(
@@ -760,38 +776,35 @@ def build_parser():
         metavar="STREAMS",
         type=comma_list,
         help="""
-        Fine tune the ``best`` and ``worst`` stream name synonyms by excluding unwanted streams.
+        Fine tune the `best` and `worst` stream name synonyms by excluding unwanted streams.
 
-        If all of the available streams get excluded, ``best`` and ``worst`` will become
-        inaccessible and new special stream synonyms ``best-unfiltered`` and ``worst-unfiltered``
+        If all of the available streams get excluded, `best` and `worst` will become
+        inaccessible and new special stream synonyms `best-unfiltered` and `worst-unfiltered`
         can be used as a fallback selection method.
 
         Uses a filter expression in the format:
 
           [operator]<value>
 
-        Valid operators are ``>``, ``>=``, ``<`` and ``<=``. If no operator is specified then
+        Valid operators are `>`, `>=`, `<` and `<=`. If no operator is specified then
         equality is tested.
 
         For example this will exclude streams ranked higher than "480p":
 
-          ">480p"
+          --stream-sorting-excludes ">480p"
 
         Multiple filters can be used by separating each expression with a comma.
 
         For example this will exclude streams from two quality types:
 
-          ">480p,>medium"
+          --stream-sorting-excludes ">480p,>medium"
 
         """
     )
 
     transport = parser.add_argument_group("Stream transport options")
-    transport_hds = transport.add_argument_group("HDS options")
-    transport_hls = transport.add_argument_group("HLS options")
-    transport_rtmp = transport.add_argument_group("RTMP options")
-    transport_subprocess = transport.add_argument_group("Subprocess options")
-    transport_ffmpeg = transport.add_argument_group("FFmpeg options")
+    transport_hls = parser.add_argument_group("HLS options", parent=transport)
+    transport_ffmpeg = parser.add_argument_group("FFmpeg options", parent=transport)
 
     transport.add_argument(
         "--ringbuffer-size",
@@ -813,7 +826,7 @@ def build_parser():
         Default is "16M".
 
         Note: A smaller size is recommended on lower end systems (such as Raspberry Pi) when playing stream types that require
-        some extra processing (such as HDS) to avoid unnecessary background processing.
+        some extra processing to avoid unnecessary background processing.
         """
     )
     transport.add_argument(
@@ -823,7 +836,7 @@ def build_parser():
         help="""
         How many attempts should be done to download each segment before giving up.
 
-        This applies to all different kinds of segmented stream types, such as DASH, HDS, HLS, etc.
+        This applies to all different kinds of segmented stream types, such as DASH, HLS, etc.
 
         Default is 3.
         """
@@ -833,9 +846,9 @@ def build_parser():
         type=num(int, max=10),
         metavar="THREADS",
         help="""
-        The size of the thread pool used to download segments. Minimum value is 1 and maximum is 10.
+        The size of the thread pool used to download segments. Minimum value is `1` and maximum is `10`.
 
-        This applies to all different kinds of segmented stream types, such as DASH, HDS, HLS, etc.
+        This applies to all different kinds of segmented stream types, such as DASH, HLS, etc.
 
         Default is 1.
         """
@@ -847,7 +860,7 @@ def build_parser():
         help="""
         Segment connect and read timeout.
 
-        This applies to all different kinds of segmented stream types, such as DASH, HDS, HLS, etc.
+        This applies to all different kinds of segmented stream types, such as DASH, HLS, etc.
 
         Default is 10.0.
         """
@@ -859,7 +872,7 @@ def build_parser():
         help="""
         Timeout for reading data from streams.
 
-        This applies to all different kinds of stream types, such as DASH, HDS, HLS, HTTP, RTMP, etc.
+        This applies to all different kinds of stream types, such as DASH, HLS, HTTP, etc.
 
         Default is 60.0.
         """
@@ -874,21 +887,6 @@ def build_parser():
         """
     )
 
-    transport_hds.add_argument(
-        "--hds-live-edge",
-        type=num(float, min=0),
-        metavar="SECONDS",
-        help="""
-        The time live HDS streams will start from the edge of the stream.
-
-        Default is 10.0.
-        """
-    )
-    transport_hds.add_argument("--hds-segment-attempts", help=argparse.SUPPRESS)
-    transport_hds.add_argument("--hds-segment-threads", help=argparse.SUPPRESS)
-    transport_hds.add_argument("--hds-segment-timeout", help=argparse.SUPPRESS)
-    transport_hds.add_argument("--hds-timeout", help=argparse.SUPPRESS)
-
     transport_hls.add_argument(
         "--hls-live-edge",
         type=num(int, min=0),
@@ -909,6 +907,13 @@ def build_parser():
         --player-args.
         """
     )
+    transport_hls.add_argument(
+        "--hls-segment-stream-data",
+        action="store_true",
+        help="""
+        Immediately write segment data into output buffer while downloading.
+        """
+    )
     transport_hls.add_argument(
         "--hls-playlist-reload-attempts",
         type=num(int, min=0),
@@ -926,9 +931,9 @@ def build_parser():
         Set a custom HLS playlist reload time value, either in seconds
         or by using one of the following keywords:
 
-            segment: The duration of the last segment in the current playlist
-            live-edge: The sum of segment durations of the live edge value minus one
-            default: The playlist's target duration metadata
+        - segment: The duration of the last segment in the current playlist
+        - live-edge: The sum of segment durations of the live edge value minus one
+        - default: The playlist's target duration metadata
 
         Default is default.
         """
@@ -974,7 +979,7 @@ def build_parser():
         metavar="CODE",
         help="""
         Selects a specific audio source or sources, by language code or name,
-        when multiple audio sources are available. Can be * to download all
+        when multiple audio sources are available. Can be `*` (asterisk) to download all
         audio sources.
 
         Examples:
@@ -1023,73 +1028,19 @@ def build_parser():
     transport_hls.add_argument("--hls-segment-attempts", help=argparse.SUPPRESS)
     transport_hls.add_argument("--hls-segment-threads", help=argparse.SUPPRESS)
     transport_hls.add_argument("--hls-segment-timeout", help=argparse.SUPPRESS)
-    transport_hls.add_argument("--hls-segment-stream-data", action="store_true", help=argparse.SUPPRESS)
     transport_hls.add_argument("--hls-timeout", help=argparse.SUPPRESS)
 
     transport.add_argument("--http-stream-timeout", help=argparse.SUPPRESS)
 
-    transport_rtmp.add_argument(
-        "--rtmp-rtmpdump",
-        metavar="FILENAME",
-        help="""
-        RTMPDump is used to access RTMP streams. You can specify the
-        location of the rtmpdump executable if it is not in your PATH.
-
-        Example: "/usr/local/bin/rtmpdump"
-        """
-    )
-    transport_rtmp.add_argument(
-        "--rtmp-proxy",
-        metavar="PROXY",
-        help="""
-        A SOCKS proxy that RTMP streams will use.
-
-        Example: 127.0.0.1:9050
-        """
-    )
-    transport_rtmp.add_argument("--rtmpdump", help=argparse.SUPPRESS)
-    transport_rtmp.add_argument("--rtmp-timeout", help=argparse.SUPPRESS)
-
-    transport_subprocess.add_argument(
-        "--subprocess-cmdline",
-        action="store_true",
-        help="""
-        Print the command-line used internally to play the stream.
-
-        This is only available on RTMP streams.
-        """
-    )
-    transport_subprocess.add_argument(
-        "--subprocess-errorlog",
-        action="store_true",
-        help="""
-        Log possible errors from internal subprocesses to a temporary file. The
-        file will be saved in your systems temporary directory.
-
-        Useful when debugging rtmpdump related issues.
-        """
-    )
-    transport_subprocess.add_argument(
-        "--subprocess-errorlog-path",
-        type=str,
-        metavar="PATH",
-        help="""
-        Log the subprocess errorlog to a specific file rather than a temporary
-        file. Takes precedence over subprocess-errorlog.
-
-        Useful when debugging rtmpdump related issues.
-        """
-    )
-
     transport_ffmpeg.add_argument(
         "--ffmpeg-ffmpeg",
         metavar="FILENAME",
         help="""
         FFMPEG is used to access or mux separate video and audio streams. You
         can specify the location of the ffmpeg executable if it is not in your
-        PATH.
+        `PATH`.
 
-        Example: "/usr/local/bin/ffmpeg"
+        Example: --ffmpeg-ffmpeg "/usr/local/bin/ffmpeg"
         """
     )
     transport_ffmpeg.add_argument(
@@ -1112,40 +1063,40 @@ def build_parser():
         type=str,
         metavar="OUTFORMAT",
         help="""
-        When muxing streams, set the output format to OUTFORMAT.
+        When muxing streams, set the output format to `OUTFORMAT`.
 
         Default is "matroska".
 
-        Example: "mpegts"
+        Example: --ffmpeg-fout "mpegts"
         """
     )
     transport_ffmpeg.add_argument(
         "--ffmpeg-video-transcode",
         metavar="CODEC",
         help="""
-        When muxing streams, transcode the video to CODEC.
+        When muxing streams, transcode the video to `CODEC`.
 
         Default is "copy".
 
-        Example: "h264"
+        Example: --ffmpeg-video-transcode "h264"
         """
     )
     transport_ffmpeg.add_argument(
         "--ffmpeg-audio-transcode",
         metavar="CODEC",
         help="""
-        When muxing streams, transcode the audio to CODEC.
+        When muxing streams, transcode the audio to `CODEC`.
 
         Default is "copy".
 
-        Example: "aac"
+        Example: --ffmpeg-audio-transcode "aac"
         """
     )
     transport_ffmpeg.add_argument(
         "--ffmpeg-copyts",
         action="store_true",
         help="""
-        Forces the -copyts ffmpeg option and does not remove
+        Forces the `-copyts` ffmpeg option and does not remove
         the initial start time offset value.
         """
     )
@@ -1153,7 +1104,7 @@ def build_parser():
         "--ffmpeg-start-at-zero",
         action="store_true",
         help="""
-        Enable the -start_at_zero ffmpeg option when using copyts.
+        Enable the `-start_at_zero` ffmpeg option when using --ffmpeg-copyts.
         """
     )
 
@@ -1162,21 +1113,12 @@ def build_parser():
         "--http-proxy",
         metavar="HTTP_PROXY",
         help="""
-        A HTTP proxy to use for all HTTP requests, including WebSocket connections.
-        By default this proxy will be used for all HTTPS requests too.
+        A HTTP proxy to use for all HTTP and HTTPS requests, including WebSocket connections.
 
-        Example: "http://hostname:port/"
-        """
-    )
-    http.add_argument(
-        "--https-proxy",
-        metavar="HTTPS_PROXY",
-        help="""
-        A HTTPS capable proxy to use for all HTTPS requests.
-
-        Example: "https://hostname:port/"
+        Example: --http-proxy "http://hostname:port/"
         """
     )
+    http.add_argument("--https-proxy", help=argparse.SUPPRESS)
     http.add_argument(
         "--http-cookie",
         metavar="KEY=VALUE",
@@ -1215,7 +1157,7 @@ def build_parser():
         action="store_true",
         help="""
         Ignore HTTP settings set in the environment such as environment
-        variables (HTTP_PROXY, etc) or ~/.netrc authentication.
+        variables (`HTTP_PROXY`, etc) or `~/.netrc` authentication.
         """
     )
     http.add_argument(
@@ -1270,4 +1212,74 @@ def build_parser():
     return parser
 
 
-__all__ = ["build_parser"]
+def false(_): return False  # noqa: E704
+
+
+# The order of arguments determines if options get overridden by `Streamlink.set_option()`
+_ARGUMENT_TO_SESSIONOPTION: List[Tuple[str, str, Optional[Callable[[Any], Any]]]] = [
+    # generic arguments
+    ("locale", "locale", None),
+
+    # network arguments
+    ("interface", "interface", None),
+    ("ipv4", "ipv4", None),
+    ("ipv6", "ipv6", None),
+
+    # HTTP session arguments
+    ("https_proxy", "https-proxy", None),
+    ("http_proxy", "http-proxy", None),
+    ("http_cookie", "http-cookies", dict),
+    ("http_header", "http-headers", dict),
+    ("http_query_param", "http-query-params", dict),
+    ("http_ignore_env", "http-trust-env", false),
+    ("http_no_ssl_verify", "http-ssl-verify", false),
+    ("http_disable_dh", "http-disable-dh", None),
+    ("http_ssl_cert", "http-ssl-cert", None),
+    ("http_ssl_cert_crt_key", "http-ssl-cert", tuple),
+    ("http_timeout", "http-timeout", None),
+
+    # deprecated stream transport arguments (need to be defined first, so following ones can override values)
+    ("hls_segment_attempts", "hls-segment-attempts", None),
+    ("hls_segment_threads", "hls-segment-threads", None),
+    ("hls_segment_timeout", "hls-segment-timeout", None),
+    ("hls_timeout", "hls-timeout", None),
+    ("http_stream_timeout", "http-stream-timeout", None),
+
+    # stream transport arguments
+    ("ringbuffer_size", "ringbuffer-size", None),
+    ("mux_subtitles", "mux-subtitles", None),
+    ("stream_segment_attempts", "stream-segment-attempts", None),
+    ("stream_segment_threads", "stream-segment-threads", None),
+    ("stream_segment_timeout", "stream-segment-timeout", None),
+    ("stream_timeout", "stream-timeout", None),
+    ("hls_live_edge", "hls-live-edge", None),
+    ("hls_live_restart", "hls-live-restart", None),
+    ("hls_start_offset", "hls-start-offset", None),
+    ("hls_duration", "hls-duration", None),
+    ("hls_playlist_reload_attempts", "hls-playlist-reload-attempts", None),
+    ("hls_playlist_reload_time", "hls-playlist-reload-time", None),
+    ("hls_segment_stream_data", "hls-segment-stream-data", None),
+    ("hls_segment_ignore_names", "hls-segment-ignore-names", None),
+    ("hls_segment_key_uri", "hls-segment-key-uri", None),
+    ("hls_audio_select", "hls-audio-select", None),
+    ("ffmpeg_ffmpeg", "ffmpeg-ffmpeg", None),
+    ("ffmpeg_verbose", "ffmpeg-verbose", None),
+    ("ffmpeg_verbose_path", "ffmpeg-verbose-path", None),
+    ("ffmpeg_fout", "ffmpeg-fout", None),
+    ("ffmpeg_video_transcode", "ffmpeg-video-transcode", None),
+    ("ffmpeg_audio_transcode", "ffmpeg-audio-transcode", None),
+    ("ffmpeg_copyts", "ffmpeg-copyts", None),
+    ("ffmpeg_start_at_zero", "ffmpeg-start-at-zero", None),
+]
+
+
+def setup_session_options(session: Streamlink, args: argparse.Namespace):
+    for arg, option, mapper in _ARGUMENT_TO_SESSIONOPTION:
+        value = getattr(args, arg)
+        if value:
+            if mapper is not None:
+                value = mapper(value)
+            session.set_option(option, value)
+
+
+__all__ = ["ArgumentParser", "build_parser", "setup_session_options"]
