@@ -6,8 +6,7 @@ import platform
 import re
 import signal
 import sys
-from contextlib import closing
-from distutils.version import StrictVersion
+from contextlib import closing, suppress
 from functools import partial
 from gettext import gettext
 from itertools import chain
@@ -15,24 +14,21 @@ from pathlib import Path
 from time import sleep
 from typing import Any, Dict, Iterator, List, Optional, Type, Union
 
-import requests
-
 import streamlink.logger as logger
 from streamlink import NoPluginError, PluginError, StreamError, Streamlink, __version__ as streamlink_version
-from streamlink.cache import Cache
 from streamlink.compat import is_win32
 from streamlink.exceptions import FatalPluginError
 from streamlink.plugin import Plugin, PluginOptions
 from streamlink.stream.stream import Stream, StreamIO
 from streamlink.utils.named_pipe import NamedPipe
-from streamlink_cli.argparser import ArgumentParser, build_parser
+from streamlink_cli.argparser import ArgumentParser, build_parser, setup_session_options
 from streamlink_cli.compat import DeprecatedPath, importlib_metadata, stdout
 from streamlink_cli.console import ConsoleOutput, ConsoleUserInputRequester
 from streamlink_cli.constants import CONFIG_FILES, DEFAULT_STREAM_METADATA, LOG_DIR, PLUGIN_DIRS, STREAM_SYNONYMS
 from streamlink_cli.output import FileOutput, PlayerOutput
-from streamlink_cli.utils import Formatter, HTTPServer, datetime, ignored
+from streamlink_cli.utils import Formatter, HTTPServer, datetime
 from streamlink_cli.utils.progress import Progress
-from streamlink_cli.utils.terminal import TerminalOutput
+from streamlink_cli.utils.versioncheck import check_version
 
 
 ACCEPTABLE_ERRNO = (errno.EPIPE, errno.EINVAL, errno.ECONNRESET)
@@ -197,6 +193,7 @@ def output_stream_http(
     initial_streams: Dict[str, Stream],
     formatter: Formatter,
     external: bool = False,
+    continuous: bool = True,
     port: int = 0,
 ):
     """Continuously output the stream over HTTP."""
@@ -229,7 +226,7 @@ def output_stream_http(
 
         log.info("Starting server, access with one of:")
         for url in server.urls:
-            log.info(" " + url)
+            log.info(f" {url}")
 
     initial_streams_used = False
     for req in iter_http_requests(server, player):
@@ -267,6 +264,9 @@ def output_stream_http(
             log.debug("Writing stream to player")
             read_stream(stream_fd, server, prebuffer, formatter)
 
+        if not continuous:
+            break
+
         server.close(True)
 
     if player:
@@ -394,7 +394,10 @@ def read_stream(stream, output, prebuffer, formatter: Formatter, chunk_size=8192
         iter(partial(stream.read, chunk_size), b"")
     )
     if show_progress or show_record_progress:
-        progress = Progress(output=TerminalOutput(sys.stderr))
+        progress = Progress(
+            sys.stderr,
+            output.filename or output.record.filename,
+        )
         stream_iterator = progress.iter(stream_iterator)
 
     try:
@@ -456,12 +459,11 @@ def handle_stream(plugin: Plugin, streams: Dict[str, Stream], stream_name: str)
         except TypeError:
             console.exit("The stream specified cannot be translated to a URL")
 
-    # Output the stream
     else:
         # Find any streams with a '_alt' suffix and attempt
         # to use these in case the main stream is not usable.
-        alt_streams = list(filter(lambda k: stream_name + "_alt" in k,
-                                  sorted(streams.keys())))
+        alt_streams = list(filter(lambda k: f"{stream_name}_alt" in k, sorted(streams.keys())))
+
         file_output = args.output or args.stdout
 
         formatter = get_formatter(plugin)
@@ -474,8 +476,14 @@ def handle_stream(plugin: Plugin, streams: Dict[str, Stream], stream_name: str)
                 log.info(f"Opening stream: {stream_name} ({stream_type})")
                 success = output_stream_passthrough(stream, formatter)
             elif args.player_external_http:
-                return output_stream_http(plugin, streams, formatter, external=True,
-                                          port=args.player_external_http_port)
+                return output_stream_http(
+                    plugin,
+                    streams,
+                    formatter,
+                    external=True,
+                    continuous=args.player_external_http_continuous,
+                    port=args.player_external_http_port,
+                )
             elif args.player_continuous_http and not file_output:
                 return output_stream_http(plugin, streams, formatter)
             else:
@@ -581,10 +589,10 @@ def handle_url():
     """
 
     try:
-        pluginclass, resolved_url = streamlink.resolve_url(args.url)
-        setup_plugin_options(streamlink, pluginclass)
-        plugin = pluginclass(resolved_url)
-        log.info(f"Found matching plugin {plugin.module} for URL {args.url}")
+        pluginname, pluginclass, resolved_url = streamlink.resolve_url(args.url)
+        setup_plugin_options(streamlink, pluginname, pluginclass)
+        plugin = pluginclass(streamlink, resolved_url)
+        log.info(f"Found matching plugin {pluginname} for URL {args.url}")
 
         if args.retry_max or args.retry_streams:
             retry_streams = 1
@@ -659,7 +667,7 @@ def load_plugins(dirs: List[Path], showwarning: bool = True):
         if directory.is_dir():
             success = streamlink.load_plugins(str(directory))
             if success and type(directory) is DeprecatedPath:
-                log.info(f"Loaded plugins from deprecated path, see CLI docs for how to migrate: {directory}")
+                log.warning(f"Loaded plugins from deprecated path, see CLI docs for how to migrate: {directory}")
         elif showwarning:
             log.warning(f"Plugin path {directory} does not exist or is not a directory!")
 
@@ -690,27 +698,30 @@ def setup_config_args(parser, ignore_unknown=False):
 
     if args.config:
         # We want the config specified last to get highest priority
-        for config_file in map(lambda path: Path(path).expanduser(), reversed(args.config)):
-            if config_file.is_file():
-                config_files.append(config_file)
+        config_files.extend(
+            config_file
+            for config_file in map(lambda path: Path(path).expanduser(), reversed(args.config))
+            if config_file.is_file()
+        )
+
     else:
         # Only load first available default config
         for config_file in filter(lambda path: path.is_file(), CONFIG_FILES):
             if type(config_file) is DeprecatedPath:
-                log.info(f"Loaded config from deprecated path, see CLI docs for how to migrate: {config_file}")
+                log.warning(f"Loaded config from deprecated path, see CLI docs for how to migrate: {config_file}")
             config_files.append(config_file)
             break
 
     if streamlink and args.url:
         # Only load first available plugin config
-        with ignored(NoPluginError):
-            pluginclass, resolved_url = streamlink.resolve_url(args.url)
+        with suppress(NoPluginError):
+            pluginname, pluginclass, resolved_url = streamlink.resolve_url(args.url)
             for config_file in CONFIG_FILES:
-                config_file = config_file.with_name(f"{config_file.name}.{pluginclass.module}")
+                config_file = config_file.with_name(f"{config_file.name}.{pluginname}")
                 if not config_file.is_file():
                     continue
                 if type(config_file) is DeprecatedPath:
-                    log.info(f"Loaded plugin config from deprecated path, see CLI docs for how to migrate: {config_file}")
+                    log.warning(f"Loaded plugin config from deprecated path, see CLI docs for how to migrate: {config_file}")
                 config_files.append(config_file)
                 break
 
@@ -725,42 +736,6 @@ def setup_signals():
     signal.signal(signal.SIGTERM, signal.default_int_handler)
 
 
-def setup_http_session():
-    """Sets the global HTTP settings, such as proxy and headers."""
-    if args.http_proxy:
-        streamlink.set_option("http-proxy", args.http_proxy)
-
-    if args.https_proxy:
-        streamlink.set_option("https-proxy", args.https_proxy)
-
-    if args.http_cookie:
-        streamlink.set_option("http-cookies", dict(args.http_cookie))
-
-    if args.http_header:
-        streamlink.set_option("http-headers", dict(args.http_header))
-
-    if args.http_query_param:
-        streamlink.set_option("http-query-params", dict(args.http_query_param))
-
-    if args.http_ignore_env:
-        streamlink.set_option("http-trust-env", False)
-
-    if args.http_no_ssl_verify:
-        streamlink.set_option("http-ssl-verify", False)
-
-    if args.http_disable_dh:
-        streamlink.set_option("http-disable-dh", True)
-
-    if args.http_ssl_cert:
-        streamlink.set_option("http-ssl-cert", args.http_ssl_cert)
-
-    if args.http_ssl_cert_crt_key:
-        streamlink.set_option("http-ssl-cert", tuple(args.http_ssl_cert_crt_key))
-
-    if args.http_timeout:
-        streamlink.set_option("http-timeout", args.http_timeout)
-
-
 def setup_plugins(extra_plugin_dir=None):
     """Loads any additional plugins."""
     load_plugins(PLUGIN_DIRS, showwarning=False)
@@ -776,84 +751,6 @@ def setup_streamlink():
     streamlink = Streamlink({"user-input-requester": ConsoleUserInputRequester(console)})
 
 
-def setup_options():
-    """Sets Streamlink options."""
-    if args.interface:
-        streamlink.set_option("interface", args.interface)
-    if args.ipv4:
-        streamlink.set_option("ipv4", args.ipv4)
-    if args.ipv6:
-        streamlink.set_option("ipv6", args.ipv6)
-
-    if args.ringbuffer_size:
-        streamlink.set_option("ringbuffer-size", args.ringbuffer_size)
-    if args.mux_subtitles:
-        streamlink.set_option("mux-subtitles", args.mux_subtitles)
-
-    if args.hls_live_edge:
-        streamlink.set_option("hls-live-edge", args.hls_live_edge)
-    if args.hls_segment_stream_data:
-        streamlink.set_option("hls-segment-stream-data", args.hls_segment_stream_data)
-
-    if args.hls_playlist_reload_attempts:
-        streamlink.set_option("hls-playlist-reload-attempts", args.hls_playlist_reload_attempts)
-    if args.hls_playlist_reload_time:
-        streamlink.set_option("hls-playlist-reload-time", args.hls_playlist_reload_time)
-    if args.hls_segment_ignore_names:
-        streamlink.set_option("hls-segment-ignore-names", args.hls_segment_ignore_names)
-    if args.hls_segment_key_uri:
-        streamlink.set_option("hls-segment-key-uri", args.hls_segment_key_uri)
-    if args.hls_audio_select:
-        streamlink.set_option("hls-audio-select", args.hls_audio_select)
-    if args.hls_start_offset:
-        streamlink.set_option("hls-start-offset", args.hls_start_offset)
-    if args.hls_duration:
-        streamlink.set_option("hls-duration", args.hls_duration)
-    if args.hls_live_restart:
-        streamlink.set_option("hls-live-restart", args.hls_live_restart)
-
-    # deprecated
-    if args.hls_segment_attempts:
-        streamlink.set_option("hls-segment-attempts", args.hls_segment_attempts)
-    if args.hls_segment_threads:
-        streamlink.set_option("hls-segment-threads", args.hls_segment_threads)
-    if args.hls_segment_timeout:
-        streamlink.set_option("hls-segment-timeout", args.hls_segment_timeout)
-    if args.hls_timeout:
-        streamlink.set_option("hls-timeout", args.hls_timeout)
-    if args.http_stream_timeout:
-        streamlink.set_option("http-stream-timeout", args.http_stream_timeout)
-
-    # generic stream- arguments take precedence over deprecated stream-type arguments
-    if args.stream_segment_attempts:
-        streamlink.set_option("stream-segment-attempts", args.stream_segment_attempts)
-    if args.stream_segment_threads:
-        streamlink.set_option("stream-segment-threads", args.stream_segment_threads)
-    if args.stream_segment_timeout:
-        streamlink.set_option("stream-segment-timeout", args.stream_segment_timeout)
-    if args.stream_timeout:
-        streamlink.set_option("stream-timeout", args.stream_timeout)
-
-    if args.ffmpeg_ffmpeg:
-        streamlink.set_option("ffmpeg-ffmpeg", args.ffmpeg_ffmpeg)
-    if args.ffmpeg_verbose:
-        streamlink.set_option("ffmpeg-verbose", args.ffmpeg_verbose)
-    if args.ffmpeg_verbose_path:
-        streamlink.set_option("ffmpeg-verbose-path", args.ffmpeg_verbose_path)
-    if args.ffmpeg_fout:
-        streamlink.set_option("ffmpeg-fout", args.ffmpeg_fout)
-    if args.ffmpeg_video_transcode:
-        streamlink.set_option("ffmpeg-video-transcode", args.ffmpeg_video_transcode)
-    if args.ffmpeg_audio_transcode:
-        streamlink.set_option("ffmpeg-audio-transcode", args.ffmpeg_audio_transcode)
-    if args.ffmpeg_copyts:
-        streamlink.set_option("ffmpeg-copyts", args.ffmpeg_copyts)
-    if args.ffmpeg_start_at_zero:
-        streamlink.set_option("ffmpeg-start-at-zero", args.ffmpeg_start_at_zero)
-
-    streamlink.set_option("locale", args.locale)
-
-
 def setup_plugin_args(session: Streamlink, parser: ArgumentParser):
     """Sets Streamlink plugin options."""
 
@@ -862,7 +759,7 @@ def setup_plugin_args(session: Streamlink, parser: ArgumentParser):
         defaults = {}
         group = parser.add_argument_group(pname.capitalize(), parent=plugin_args)
 
-        for parg in plugin.arguments:
+        for parg in plugin.arguments or []:
             if not parg.is_global:
                 group.add_argument(parg.argument_name(pname), **parg.options)
                 defaults[parg.dest] = parg.default
@@ -882,17 +779,19 @@ def setup_plugin_args(session: Streamlink, parser: ArgumentParser):
         plugin.options = PluginOptions(defaults)
 
 
-def setup_plugin_options(session: Streamlink, plugin: Type[Plugin]):
+def setup_plugin_options(session: Streamlink, pluginname: str, pluginclass: Type[Plugin]):
     """Sets Streamlink plugin options."""
-    pname = plugin.module
+    if pluginclass.arguments is None:
+        return
+
     required = {}
 
-    for parg in plugin.arguments:
+    for parg in pluginclass.arguments:
         if parg.options.get("help") == argparse.SUPPRESS:
             continue
 
-        value = getattr(args, parg.dest if parg.is_global else parg.namespace_dest(pname))
-        session.set_plugin_option(pname, parg.dest, value)
+        value = getattr(args, parg.dest if parg.is_global else parg.namespace_dest(pluginname))
+        session.set_plugin_option(pluginname, parg.dest, value)
 
         if not parg.is_global:
             if parg.required:
@@ -900,18 +799,18 @@ def setup_plugin_options(session: Streamlink, plugin: Type[Plugin]):
             # if the value is set, check to see if any of the required arguments are not set
             if parg.required or value:
                 try:
-                    for rparg in plugin.arguments.requires(parg.name):
+                    for rparg in pluginclass.arguments.requires(parg.name):
                         required[rparg.name] = rparg
                 except RuntimeError:
-                    log.error(f"{pname} plugin has a configuration error and the arguments cannot be parsed")
+                    log.error(f"{pluginname} plugin has a configuration error and the arguments cannot be parsed")
                     break
 
     if required:
         for req in required.values():
-            if not session.get_plugin_option(pname, req.dest):
-                prompt = f"{req.prompt or f'Enter {pname} {req.name}'}: "
+            if not session.get_plugin_option(pluginname, req.dest):
+                prompt = f"{req.prompt or f'Enter {pluginname} {req.name}'}: "
                 session.set_plugin_option(
-                    pname,
+                    pluginname,
                     req.dest,
                     console.askpass(prompt) if req.sensitive else console.ask(prompt)
                 )
@@ -957,14 +856,14 @@ def log_current_versions():
         log.debug(f" {name}: {version}")
 
 
-def log_current_arguments(session, parser):
+def log_current_arguments(session: Streamlink, parser: argparse.ArgumentParser):
     global args
     if not logger.root.isEnabledFor(logging.DEBUG):
         return
 
     sensitive = set()
     for pname, plugin in session.plugins.items():
-        for parg in plugin.arguments:
+        for parg in plugin.arguments or []:
             if parg.sensitive:
                 sensitive.add(parg.argument_name(pname))
 
@@ -981,33 +880,6 @@ def log_current_arguments(session, parser):
             log.debug(f" {name}={value if name not in sensitive else '*' * 8}")
 
 
-def check_version(force=False):
-    cache = Cache(filename="cli.json")
-    latest_version = cache.get("latest_version")
-
-    if force or not latest_version:
-        res = requests.get("https://pypi.python.org/pypi/streamlink/json")
-        data = res.json()
-        latest_version = data.get("info").get("version")
-        cache.set("latest_version", latest_version, (60 * 60 * 24))
-
-    version_info_printed = cache.get("version_info_printed")
-    if not force and version_info_printed:
-        return
-
-    installed_version = StrictVersion(streamlink.version)
-    latest_version = StrictVersion(latest_version)
-
-    if latest_version > installed_version:
-        log.info(f"A new version of Streamlink ({latest_version}) is available!")
-        cache.set("version_info_printed", True, (60 * 60 * 6))
-    elif force:
-        log.info(f"Your Streamlink version ({installed_version}) is up to date!")
-
-    if force:
-        sys.exit()
-
-
 def setup_logger_and_console(stream=sys.stdout, filename=None, level="info", json=False):
     global console
 
@@ -1064,19 +936,23 @@ def main():
     log_level = args.loglevel if not silent_log else "none"
     logger.root.setLevel(log_level)
 
-    setup_http_session()
-
     log_root_warning()
     log_current_versions()
     log_current_arguments(streamlink, parser)
 
+    setup_session_options(streamlink, args)
+
     setup_signals()
 
     if args.version_check or args.auto_version_check:
-        with ignored(Exception):
+        try:
             check_version(force=args.version_check)
+        except KeyboardInterrupt:
+            error_code = 130
 
-    if args.help:
+    if args.version_check:
+        pass
+    elif args.help:
         parser.print_help()
     elif args.plugins:
         print_plugins()
@@ -1096,7 +972,6 @@ def main():
             error_code = 130
     elif args.url:
         try:
-            setup_options()
             handle_url()
         except KeyboardInterrupt:
             # Close output
